////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLE_PLUGIN_H_INCLUDED
#define TRIANGLE_PLUGIN_H_INCLUDED

#include <GtfMessenger/GtfMsgExtReceiver.h>
#include "TriangleDesc.h"

#include <gtf/scdr/sdk/SDK.h>

//!
//! \brief Implementation of the GTF plugin.
//!
class TrianglePlugin : public GtfMsgExtReceiver
{
public:

    TrianglePlugin();

    ~TrianglePlugin();

    //!
    //! \brief Called by GTF at startup of the plugin.
    //!
    //! \param[in] loader_ The GTF plugin loader.
    //!
    void startup(GtfPluginLoader& loader_);

    //!
    //! \brief Called by GTF at shutdown of the plugin.
    //!
    void shutdown();

    //!
    //! \brief Called by GTF when a new runlevel is reached.
    //!
    //! \param[in] runlevel_ The actual run level.
    //! \param[in] direction_    +1 for up (startup), -1 for down (shutdown).
    //!
    //! \return The next requested runlevel.
    //!
    //! \details Callback from GTF, the user has the possibility to initialize
    //!          the widget at defined points (runlevel) at startup.
    //!
    uint16_t HandleRunlevel(uint16_t const runlevel_, int8_t const direction_);

    //!
    //! \brief Notice a new component interface.
    //!
    //! \param interfaceId_[in]   Unique numerical ID of the interface.
    //! \param version_[in]       Version number of the interface.
    //! \param validRunlevel_[in] The lowest runlevel required by the interface implementing object to work properly.
    //! \param object_[in]        Pointer to an object, that implements the interface.
    //!
    void HandleInterface(uint32_t const interfaceId_, uint32_t const version_, uint16_t const validRunlevel_, void * const object_);

    //!
    //! \brief Process an incoming message.
    //! \param msg_ The handle of the received message.
    //!
    void HandleMessage(GtfMsgHandle const msg_);

private:

    void registerRenderer();

    //!
    //! \brief Pointer to the SCDR SDK interface.
    //!
    gtf::scdr::sdk::SDK* sdk;

    //!
    //! \brief Interface passed to the create callback of the custom renderer.
    //! \details Provides functionality to resolve properties.
    //!
    gtf::scdr::sdk::WidgetRendererRegistration* registration;
    
    bool initialized;
};

#endif // TRIANGLE_PLUGIN_H_INCLUDED